@@ -509,7 +509,7 @@ class _BaseArchetype:
         Args:
             index: The index of the entity to reference.
             check: If true, raises an error if the entity at the index is not of this archetype or of a subclass of
-            this archetype. If false, no validation is performed.
+                   this archetype. If false, no validation is performed.
 
         Returns:
             The entity at the given index.
@@ -528,7 +528,7 @@ class _BaseArchetype:
         Args:
             index: The index of the entity to check.
             strict: If true, only returns true if the entity is exactly of this archetype. If false, also returns true
-            if the entity is of a subclass of this archetype.
+                    if the entity is of a subclass of this archetype.
 
         Returns:
             Whether the entity at the given index is of this archetype.
@@ -647,25 +647,28 @@ class _BaseArchetype:
         # Archetype parents would have already initialized relevant fields, so only consider the current class
         # and mixins that were not already included via an archetype parent
         mro_excluding_archetype_parents = [entry for entry in cls.mro() if entry not in mro_from_archetype_parents]
-        field_specifiers = get_field_specifiers(
-            cls,
-            skip={
-                "id",
-                "key",
-                "name",
-                "life",
-                "is_scored",
-                "_key_",
-                "_is_scored_",
-                "_derived_base_",
-                "_is_derived_",
-                "_default_callbacks_",
-                "_callbacks_",
-                "_field_init_done",
-                "_is_concrete_archetype_",
-            },
-            included_classes=mro_excluding_archetype_parents,
-        ).items()
+        try:
+            field_specifiers = get_field_specifiers(
+                cls,
+                skip={
+                    "id",
+                    "key",
+                    "name",
+                    "life",
+                    "is_scored",
+                    "_key_",
+                    "_is_scored_",
+                    "_derived_base_",
+                    "_is_derived_",
+                    "_default_callbacks_",
+                    "_callbacks_",
+                    "_field_init_done",
+                    "_is_concrete_archetype_",
+                },
+                included_classes=mro_excluding_archetype_parents,
+            ).items()
+        except Exception as e:
+            raise TypeError(f"Error while processing fields of {cls.__name__}: {e}") from e
         if not hasattr(cls, "_imported_fields_"):
             cls._imported_fields_ = {}
         else:
@@ -707,14 +710,14 @@ class _BaseArchetype:
                             pass
                         else:
                             raise TypeError(
-                                f"Unexpected multiple field annotations for '{name}', "
+                                f"Unexpected multiple annotations for field '{name}' of {cls.__name__}, "
                                 f"expected exactly one of imported, exported, entity_memory, or shared_memory"
                             )
                     else:
                         field_info = metadata
             if field_info is None:
                 raise TypeError(
-                    f"Missing field annotation for '{name}', "
+                    f"Missing annotation for '{name}' of {cls.__name__}, "
                     f"expected exactly one of imported, exported, entity_memory, or shared_memory"
                 )
             if (
@@ -724,7 +727,10 @@ class _BaseArchetype:
                 or name in cls._shared_memory_fields_
             ):
                 raise ValueError(f"Field '{name}' is already defined in a superclass")
-            field_type = validate_concrete_type(value.__args__[0])
+            try:
+                field_type = validate_concrete_type(value.__args__[0])
+            except Exception as e:
+                raise TypeError(f"Error in field '{name}' of {cls.__name__}: {e}") from e
             match field_info.storage:
                 case _StorageType.IMPORTED:
                     cls._imported_fields_[name] = _ArchetypeField(
@@ -1349,11 +1355,14 @@ class EntityRef[A: _BaseArchetype](Record):
 
         Args:
             check: If true, raises an error if the referenced entity is not of this archetype or of a subclass of
-            this archetype. If false, no validation is performed.
+                   this archetype. If false, no validation is performed.
 
         Returns:
             The entity this reference points to.
         """
+        assert self.archetype() != Any, (
+            "Cannot get entity of unknown (Any) archetype. Use with_archetype() first or use get_as()."
+        )
         if ref := getattr(self, "_ref_", None):
             return ref
         return self.archetype().at(self.index, check=check)
@@ -1370,11 +1379,14 @@ class EntityRef[A: _BaseArchetype](Record):
 
         Args:
             strict: If true, only returns true if the entity is exactly of this archetype. If false, also returns true
-            if the entity is of a subclass of this archetype.
+                    if the entity is of a subclass of this archetype.
 
         Returns:
             Whether the entity at the given index is of this archetype.
         """
+        assert self.archetype() != Any, (
+            "Cannot use archetype_matches with unknown (Any) archetype. Use with_archetype() first."
+        )
         return self.archetype().is_at(self.index, strict=strict)
 
     def _to_list_(self, level_refs: dict[Any, str] | None = None) -> list[DataValue | str]:
@@ -1410,6 +1422,11 @@ class EntityRef[A: _BaseArchetype](Record):
             result._ref_ = value._ref_
         return result
 
+    @classmethod
+    def _validate_parameterized_(cls):
+        if not issubclass(cls.archetype(), _BaseArchetype) and cls.archetype() is not Any:
+            raise TypeError("EntityRef type parameter must be an Archetype or Any")
+
 
 class StandardArchetypeName(StrEnum):
     """Standard archetype names."""
