@@ -1,7 +1,10 @@
 from __future__ import annotations
 
-from typing import Self
+from collections.abc import Callable
+from typing import Any, Protocol, Self
 
+from sonolus.backend.visitor import compile_and_call
+from sonolus.script.archetype import AnyArchetype, EntityRef
 from sonolus.script.array import Array
 from sonolus.script.array_like import ArrayLike, get_positive_index
 from sonolus.script.debug import error
@@ -87,6 +90,8 @@ class Pair[T, U](Record):
 class VarArray[T, Capacity](Record, ArrayLike[T]):
     """An array with a variable size and fixed maximum capacity.
 
+    Supports negative indexes.
+
     Usage:
         ```python
         VarArray[T, Capacity].new()  # Create a new empty array
@@ -125,6 +130,8 @@ class VarArray[T, Capacity](Record, ArrayLike[T]):
     def __getitem__(self, item) -> T:
         """Return the element at the given index.
 
+        Supports negative indexes.
+
         The returned value continues to be part of the array.
         Future modifications to the array will affect the returned value.
 
@@ -183,7 +190,7 @@ class VarArray[T, Capacity](Record, ArrayLike[T]):
         assert self._size + len(values) <= len(self._array), "Array is full"
         i = 0
         while i < len(values):
-            self._array.set_unchecked(self._size + i, values[i])
+            self._array.set_unchecked(self._size + i, values.get_unchecked(i))
             i += 1
         self._size += len(values)
 
@@ -310,6 +317,8 @@ class VarArray[T, Capacity](Record, ArrayLike[T]):
 class ArrayPointer[T](Record, ArrayLike[T]):
     """An array defined by a size and pointer to the first element.
 
+    Supports negative indexes.
+
     This is intended to be created internally and improper use may result in hard to debug issues.
 
     Usage:
@@ -433,6 +442,56 @@ class ArraySet[T, Capacity](Record):
         self._values.clear()
 
 
+class FrozenNumSet[Size](Record):
+    _values: Array[Num, Size]
+
+    @classmethod
+    @meta_fn
+    def of(cls, *values: Num) -> Self:
+        if ctx():
+            try:
+                num_values = [Num._accept_(v) for v in values]
+            except TypeError:
+                raise TypeError("Only sets of numeric values are supported") from None
+            if all(v._is_py_() for v in num_values):
+                const_values = [v._as_py_() for v in num_values]
+                arr = Array[Num, len(const_values)]._with_value([Num(v) for v in sorted(const_values)])
+                return cls(arr)
+            else:
+                arr = Array[Num, len(values)](*values)
+                compile_and_call(arr.sort)
+        else:
+            arr = Array[Num, len(values)](*sorted(values))
+        return cls(arr)
+
+    def __len__(self) -> int:
+        return len(self._values)
+
+    def __contains__(self, value: Num) -> bool:
+        if len(self) < 8:
+            return value in self._as_tuple()
+        else:
+            left = 0
+            right = len(self) - 1
+            while left <= right:
+                mid = (left + right) // 2
+                mid_value = self._values.get_unchecked(mid)
+                if mid_value == value:
+                    return True
+                elif mid_value < value:
+                    left = mid + 1
+                else:
+                    right = mid - 1
+            return False
+
+    def __iter__(self) -> SonolusIterator[Num]:
+        return self._values.__iter__()
+
+    @meta_fn
+    def _as_tuple(self) -> tuple[Num, ...]:
+        return tuple(self._values.get_unchecked(i) for i in range(Num._accept_(len(self))._as_py_()))
+
+
 class _ArrayMapEntry[K, V](Record):
     key: K
     value: V
@@ -637,3 +696,255 @@ class _ArrayMapEntryIterator[K, V, Capacity](Record, SonolusIterator):
             self._index += 1
             return Some(result)
         return Nothing
+
+
+class _LinkedListNodeRef[TKey, TValue](Protocol):
+    def get_value(self) -> TValue: ...
+
+    def get_next(self) -> Self: ...
+
+    def set_next(self, next_node: Self): ...
+
+    def set_prev(self, prev_node: Self):
+        # No-op for singly linked lists
+        return
+
+    def is_present(self) -> bool: ...
+
+    def set(self, other: Self): ...
+
+    def copy(self) -> Self: ...
+
+    def empty(self) -> Self: ...
+
+
+def _merge_linked_list_nodes[TNode: _LinkedListNodeRef](
+    a: TNode,
+    b: TNode,
+) -> TNode:
+    head = a.empty()
+    tail = a.empty()
+    left = a.copy()
+    right = b.copy()
+
+    while left.is_present() and right.is_present():
+        if left.get_value() <= right.get_value():
+            if not head.is_present():
+                head.set(left)
+                tail.set(left)
+            else:
+                tail.set_next(left)
+                tail.set(left)
+            left.set(left.get_next())
+        else:
+            if not head.is_present():
+                head.set(right)
+                tail.set(right)
+            else:
+                tail.set_next(right)
+                tail.set(right)
+            right.set(right.get_next())
+
+    while left.is_present():
+        if not head.is_present():
+            head.set(left)
+            tail.set(left)
+        else:
+            tail.set_next(left)
+            tail.set(left)
+        left.set(left.get_next())
+
+    while right.is_present():
+        if not head.is_present():
+            head.set(right)
+            tail.set(right)
+        else:
+            tail.set_next(right)
+            tail.set(right)
+        right.set(right.get_next())
+
+    if tail.is_present():
+        tail.set_next(a.empty())
+
+    return head
+
+
+def _merge_sort_linked_list_nodes[TNode: _LinkedListNodeRef](
+    head: TNode,
+) -> TNode:
+    # Calculate length
+    length = 0
+    node = head.copy()
+    while node.is_present():
+        length += 1
+        node.set(node.get_next())
+
+    # Trivial case
+    if length <= 1:
+        return head
+
+    # Bottom-up merge sort: start with sublists of size 1, then 2, 4, 8, etc.
+    size = 1
+    while size < length:
+        current = head.copy()
+        new_head = head.empty()
+        new_tail = head.empty()
+
+        # Process all pairs of sublists of the current size
+        while current.is_present():
+            # Extract the first sublist
+            left = current.copy()
+            prev = current.empty()
+            i = 0
+            while i < size and current.is_present():
+                prev.set(current)
+                current.set(current.get_next())
+                i += 1
+            if prev.is_present():
+                prev.set_next(prev.empty())
+
+            # We've made it to the end without a second sublist to merge, so just attach it to the end
+            if not current.is_present():
+                # Since size < length, we know a full iteration must have happened already, so new_tail is valid
+                new_tail.set_next(left)
+                break
+
+            # Extract the second sublist
+            right = current.copy()
+            prev = current.empty()
+            i = 0
+            while i < size and current.is_present():
+                prev.set(current)
+                current.set(current.get_next())
+                i += 1
+            if prev.is_present():
+                prev.set_next(prev.empty())
+
+            merged = _merge_linked_list_nodes(left, right)
+
+            # Append the merged result
+            if not new_head.is_present():
+                new_head.set(merged)
+                new_tail.set(merged)
+            else:
+                new_tail.set_next(merged)
+
+            # Move tail to the end of the merged section
+            while new_tail.get_next().is_present():
+                new_tail.set(new_tail.get_next())
+
+        # Update head for the next iteration
+        head.set(new_head)
+        size *= 2
+
+    return head
+
+
+class _EntityNodeRef[Archetype, GetValue, GetNextRef, GetPrevRef](Record):
+    index: int
+
+    def get_value(self) -> Any:
+        return self.get_value_fn(self.archetype.at(self.index))
+
+    def get_next(self) -> _EntityNodeRef:
+        next_ref = self.get_next_ref_fn(self.archetype.at(self.index))
+        return self.with_index(next_ref.index)
+
+    def set_next(self, next_node: _EntityNodeRef):
+        entity = self.archetype.at(self.index)
+        next_ref = self.get_next_ref_fn(entity)
+        next_ref.index = next_node.index
+
+    def set_prev(self, prev_node: _EntityNodeRef):
+        if self.get_prev_ref_fn is not None:
+            entity = self.archetype.at(self.index)
+            prev_ref = self.get_prev_ref_fn(entity)
+            prev_ref.index = prev_node.index
+
+    def is_present(self) -> bool:
+        return self.index > 0
+
+    def set(self, other: _EntityNodeRef):
+        self.index = other.index
+
+    def copy(self) -> _EntityNodeRef:
+        return self.with_index(self.index)
+
+    def empty(self) -> _EntityNodeRef:
+        return self.with_index(0)
+
+    def with_index(self, index: int) -> _EntityNodeRef:
+        return _EntityNodeRef[
+            self.archetype,
+            self.get_value_fn,
+            self.get_next_ref_fn,
+            self.get_prev_ref_fn,
+        ](index)
+
+    @property
+    def archetype(self):
+        return self.type_var_value(Archetype)
+
+    @property
+    def get_value_fn(self):
+        return self.type_var_value(GetValue)
+
+    @property
+    def get_next_ref_fn(self):
+        return self.type_var_value(GetNextRef)
+
+    @property
+    def get_prev_ref_fn(self):
+        return self.type_var_value(GetPrevRef)
+
+
+def sort_linked_entities[T: AnyArchetype](
+    head_ref: EntityRef[T],
+    /,
+    *,
+    get_value: Callable[[T], Any],
+    get_next_ref: Callable[[T], EntityRef[T]],
+    get_prev_ref: Callable[[T], EntityRef[T]] | None = None,
+) -> EntityRef[T]:
+    """Sort a linked list of entities using merge sort.
+
+    If get_prev_ref is provided, the backward links will be updated as well.
+
+    Usage:
+        ```python
+        class MyArchetype(PlayArchetype):
+            sort_key: int
+            next: EntityRef[MyArchetype]
+
+        def sort_my_archetype(head: EntityRef[MyArchetype]) -> EntityRef[MyArchetype]:
+            return sort_linked_entities(
+                head,
+                get_value=lambda e: e.sort_key,
+                get_next_ref=lambda e: e.next,
+            )
+        ```
+
+    Args:
+        head_ref: A reference to the head of the linked list.
+        get_value: A function that takes an entity and returns the value to sort by.
+        get_next_ref: A function that takes an entity and returns a reference to the next entity.
+        get_prev_ref: An optional function that takes an entity and returns a reference to the previous entity.
+
+    Returns:
+        A reference to the head of the sorted linked list.
+    """
+    archetype = head_ref.archetype()
+
+    sorted_head_index = _merge_sort_linked_list_nodes(
+        _EntityNodeRef[archetype, get_value, get_next_ref, get_prev_ref](head_ref.index)
+    ).index
+
+    if get_prev_ref is not None:
+        current_ref = _EntityNodeRef[archetype, get_value, get_next_ref, get_prev_ref](sorted_head_index)
+        prev_ref = current_ref.empty()
+        while current_ref.is_present():
+            current_ref.set_prev(prev_ref)
+            prev_ref.set(current_ref)
+            current_ref.set(current_ref.get_next())
+
+    return EntityRef[archetype](sorted_head_index)
