@@ -27,6 +27,8 @@ def _is_num(value: Any) -> TypeGuard[Num]:
 
 @final
 class _Num(Value, metaclass=_NumMeta):
+    __slots__ = ("data",)
+
     # This works for ints, floats, and bools
     # Since we don't support complex numbers, real is equal to the original number
     __match_args__ = ("real",)
@@ -34,16 +36,10 @@ class _Num(Value, metaclass=_NumMeta):
     data: DataValue
 
     def __init__(self, data: DataValue | IRExpr):
-        if isinstance(data, complex):
-            raise TypeError("Cannot create a Num from a complex number")
         if isinstance(data, int):
             data = float(data)
         if isinstance(data, IRConst | IRPureInstr | IRGet):
             data = ExprBackingValue(data)
-        if _is_num(data):
-            raise InternalError("Cannot create a Num from a Num")
-        if not isinstance(data, BlockPlace | BackingValue | float | int | bool):
-            raise TypeError(f"Cannot create a Num from {type(data)}")
         self.data = data
 
     def __str__(self) -> str:
@@ -131,6 +127,8 @@ class _Num(Value, metaclass=_NumMeta):
                 if isinstance(self.data.block, BlockData) and not ctx().is_writable(self.data):
                     # This block is immutable in the current callback, so no need to copy it in case it changes.
                     return Num(self.data)
+            if isinstance(self.data, int | float | bool):
+                return Num(self.data)
             place = ctx().alloc(size=1)
             ctx().add_statements(IRSet(place, self.ir()))
             return Num(place)
@@ -449,6 +447,12 @@ class _Num(Value, metaclass=_NumMeta):
         return 0
 
 
+def _create_num_raw(i: int) -> Num:
+    result = object.__new__(_Num)
+    result.data = float(i)
+    return result
+
+
 if TYPE_CHECKING:
     from typing import Protocol
 
